@@ -1,12 +1,12 @@
 from compas.geometry import distance_point_point
 
-from compas_cem.optimization.constraints import Constraint
+from compas_cem.optimization.constraints import FloatConstraint
 
 
 __all__ = ["DeviationEdgeLengthConstraint"]
 
 
-class DeviationEdgeLengthConstraint(Constraint):
+class DeviationEdgeLengthConstraint(FloatConstraint):
     """
     Make a deviation edge reach a target length.
     """
@@ -14,22 +14,6 @@ class DeviationEdgeLengthConstraint(Constraint):
         # TODO: needs different serialization mechanism
         super(DeviationEdgeLengthConstraint, self).__init__(edge, length, weight)
 
-    def error(self, data):
-        """
-        The length error of a deviation edge.
-
-        Returns
-        -------
-        error : ``float``
-            The squared difference bewteen the current and the target length.
-        """
-        length_a = self.reference(data)
-        length_b = self.target()
-
-        diff = length_a - length_b
-
-        return diff * diff * self.weight
-
     def reference(self, data):
         """
         """
