@@ -124,7 +124,7 @@ class Optimizer():
 # ------------------------------------------------------------------------------
 # Solver
 # ------------------------------------------------------------------------------
-#
+
     def solve_nlopt(self, topology, algorithm, iters, eps=None):
         """
         Solve an optimization problem with NLOpt.
@@ -270,11 +270,11 @@ class Optimizer():
     def _calculate_penalty(self, eq_state):
         """
         """
-        error = 0.0
-        for goal in self.constraints.values():
-            error += goal.error(eq_state)
+        penalty = 0.0
+        for constraint in self.constraints.values():
+            penalty += constraint.penalty(eq_state)
 
-        return error
+        return penalty
 
     def _optimize_form(self, parameters, topology):
         """
@@ -302,10 +302,10 @@ class Optimizer():
             msg = "No constraints defined. Optimization not possible."
             raise ValueError(msg)
 
-
 # ------------------------------------------------------------------------------
 # Main
 # ------------------------------------------------------------------------------
 
+
 if __name__ == "__main__":
     pass
